@@ -60,6 +60,9 @@
 #ifdef BUILD_IMLIB2
 #include "imlib2.h"
 #endif /* BUILD_IMLIB2 */
+#ifdef BUILD_XSHAPE
+#include <X11/extensions/shape.h>
+#endif /* BUILD_XSHAPE */
 #endif /* BUILD_X11 */
 #ifdef BUILD_NCURSES
 #include <ncurses.h>
@@ -200,7 +203,7 @@ int top_io;
 #endif
 int top_running;
 static conky::simple_config_setting<bool> extra_newline("extra_newline", false, false);
-static volatile sig_atomic_t g_sigterm_pending, g_sighup_pending;
+static volatile sig_atomic_t g_sigterm_pending, g_sighup_pending,g_sigusr2_pending;
 
 /* Update interval */
 conky::range_config_setting<double> update_interval("update_interval", 0.0,
@@ -314,6 +317,9 @@ static void print_version(void)
 #ifdef BUILD_IMLIB2
                 << _("  * Imlib2\n")
 #endif /* BUILD_IMLIB2 */
+#ifdef HAVE_SOME_SOUNDCARD_H
+                << _("  * OSS mixer support\n")
+#endif /* HAVE_SOME_SOUNDCARD_H */
 #ifdef BUILD_MIXER_ALSA
                 << _("  * ALSA mixer support\n")
 #endif /* BUILD_MIXER_ALSA */
@@ -562,7 +568,7 @@ static bool isutf8(const char* envvar) {
 }
 
 /* UTF-8 */
-static conky::simple_config_setting<bool> utf8_mode("override_utf8_locale",
+conky::simple_config_setting<bool> utf8_mode("override_utf8_locale",
 					isutf8("LC_ALL") || isutf8("LC_CTYPE") || isutf8("LANG"), false);
 
 #endif /* BUILD_X11 */
@@ -1930,7 +1936,9 @@ static void draw_text(void)
 
 static void draw_stuff(void)
 {
+#ifndef BUILD_X11
 	static int text_offset_x, text_offset_y; /* offset for start position */
+#endif
 	text_offset_x = text_offset_y = 0;
 #ifdef BUILD_IMLIB2
 	cimlib_render(text_start_x, text_start_y, window.width, window.height);
@@ -2068,6 +2076,25 @@ static void main_loop(void)
 	sigaddset(&newmask, SIGUSR1);
 #endif
 
+#ifdef BUILD_X11
+#ifdef BUILD_XSHAPE
+	if (out_to_x.get(*state)) {
+		/* allow only decorated windows to be given mouse input */
+		int major_version, minor_version;
+		if (!XShapeQueryVersion(display, &major_version, &minor_version)) {
+			NORM_ERR("Input shapes are not supported");
+		} else {
+			if (own_window.get(*state) &&
+					(own_window_type.get(*state) != TYPE_NORMAL ||
+					 (TEST_HINT(own_window_hints.get(*state), HINT_UNDECORATED)))) {
+				XShapeCombineRectangles(display, window.window, ShapeInput, 0, 0,
+						NULL, 0, ShapeSet, Unsorted);
+			}
+		}
+	}
+#endif /* BUILD_XSHAPE */
+#endif /* BUILD_X11 */
+
 	last_update_time = 0.0;
 	next_update_time = get_time() - fmod(get_time(), active_update_interval());
 	info.looped = 0;
@@ -2445,8 +2472,23 @@ static void main_loop(void)
 		if (g_sighup_pending) {
 			g_sighup_pending = false;
 			NORM_ERR("received SIGHUP or SIGUSR1. reloading the config file.");
+
 			reload_config();
 		}
+		
+		if(g_sigusr2_pending){
+			g_sigusr2_pending = false;
+			// refresh view;
+			NORM_ERR("recieved SIGUSR2. refreshing.");
+			update_text();
+			draw_stuff();
+#ifdef BUILD_NCURSES
+			if(out_to_ncurses.get(*state)) {
+				refresh();
+				clear();
+			}
+#endif		
+		}
 
 		if (g_sigterm_pending) {
 			g_sigterm_pending = false;
@@ -3058,6 +3100,7 @@ void initialisation(int argc, char **argv) {
 	if (		sigaction(SIGINT,  &act, &oact) < 0
 			||	sigaction(SIGALRM, &act, &oact) < 0
 			||	sigaction(SIGUSR1, &act, &oact) < 0
+			||	sigaction(SIGUSR2, &act, &oact) < 0
 			||	sigaction(SIGHUP,  &act, &oact) < 0
 			||	sigaction(SIGTERM, &act, &oact) < 0) {
 		NORM_ERR("error setting signal handler: %s", strerror(errno));
@@ -3077,6 +3120,7 @@ int main(int argc, char **argv)
 	argv_copy = argv;
 	g_sigterm_pending = false;
 	g_sighup_pending = false;
+	g_sigusr2_pending = false;
 
 #ifdef BUILD_CURL
 	struct curl_global_initializer {
@@ -3204,6 +3248,8 @@ static void signal_handler(int sig)
 		case SIGUSR1:
 			g_sighup_pending = true;
 			break;
+		case SIGUSR2:
+			g_sigusr2_pending = true;
 		default:
 			/* Reaching here means someone set a signal
 			 * (SIGXXXX, signal_handler), but didn't write any code
