@@ -252,6 +252,20 @@ enum {
 #define KFLAG_FLIP(a) info.kflags ^= a
 #define KFLAG_ISSET(a) info.kflags & a
 
+#if !defined(MAX)
+#define MAX(x,y) \
+  ({ __typeof__ (x) _x = (x); \
+   __typeof__ (y) _y = (y); \
+   _x > _y ? _x : _y;})
+#endif
+
+#if !defined(MIN)
+#define MIN(x,y) \
+  ({ __typeof__ (x) _x = (x); \
+   __typeof__ (y) _y = (y); \
+   _x < _y ? _x : _y;})
+#endif
+
 /* defined in conky.c, needed by top.c */
 extern int top_cpu, top_mem, top_time;
 #ifdef BUILD_IOSTATS
@@ -298,6 +312,12 @@ void generate_text_internal(char *, int, struct text_object);
 int percent_print(char *, int, unsigned);
 void human_readable(long long, char *, int);
 
+#ifdef BUILD_X11
+
+/* UTF-8 */
+extern conky::simple_config_setting<bool> utf8_mode;
+#endif
+
 /* maximum size of config TEXT buffer, i.e. below TEXT line. */
 extern conky::range_config_setting<unsigned int> max_user_text;
 
