@@ -1,4 +1,7 @@
-/* smapi.h:  conky support for IBM Thinkpad smapi
+/* -*- mode: c++; c-basic-offset: 4; tab-width: 4; indent-tabs-mode: t -*-
+ * vim: ts=4 sw=4 noet ai cindent syntax=cpp
+ *
+ * smapi.h:  conky support for IBM Thinkpad smapi
  *
  * Copyright (C) 2007 Phil Sutter <Phil@nwl.cc>
  *
@@ -22,16 +25,11 @@
 #ifndef _SMAPI_H
 #define _SMAPI_H
 
-int smapi_bat_installed(int);
-
-char *smapi_read_str(const char *);
-int smapi_read_int(const char *);
-
-char *smapi_get_str(const char *);
-char *smapi_get_val(const char *);
-
-char *smapi_get_bat_str(int, const char *);
-int smapi_get_bat_int(int, const char *);
-char *smapi_get_bat_val(const char *);
+void print_smapi(struct text_object *, char *, int);
+uint8_t smapi_bat_percentage(struct text_object *);
+void print_smapi_bat_temp(struct text_object *, char *, int);
+void print_smapi_bat_power(struct text_object *, char *, int);
+double smapi_bat_barval(struct text_object *);
+int smapi_bat_installed(struct text_object *obj);
 
 #endif /* _SMAPI_H */
