@@ -1,4 +1,7 @@
-/* Conky, a system monitor, based on torsmo
+/* -*- mode: c++; c-basic-offset: 4; tab-width: 4; indent-tabs-mode: t -*-
+ * vim: ts=4 sw=4 noet ai cindent syntax=cpp
+ *
+ * Conky, a system monitor, based on torsmo
  *
  * Any original torsmo code is licensed under the BSD license
  *
@@ -7,7 +10,7 @@
  * Please see COPYING for details
  *
  * Copyright (c) 2004, Hannu Saransaari and Lauri Hakkarainen
- * Copyright (c) 2005-2009 Brenden Matthews, Philip Kovacs, et. al.
+ * Copyright (c) 2005-2012 Brenden Matthews, Philip Kovacs, et. al.
  *	(see AUTHORS)
  * All rights reserved.
  *
@@ -31,13 +34,12 @@
 
 #define SPECIAL_CHAR '\x01'
 
-#define MAX_GRAPH_DEPTH 512
-
 // don't use spaces in LOGGRAPH or NORMGRAPH if you change them
 #define LOGGRAPH "-l"
 #define TEMPGRAD "-t"
 
 enum special_types {
+	NONSPECIAL = 0,
 	HORIZONTAL_LINE = 1,
 	STIPPLED_HR,
 	BAR,
@@ -52,67 +54,61 @@ enum special_types {
 	VOFFSET,
 	FONT,
 	GOTO,
-	TAB,
+	TAB
 };
 
 struct special_t {
 	int type;
 	short height;
 	short width;
-	long arg;
+	double arg;
 	double *graph;
-	double graph_scale;
+	double scale;			/* maximum value */
 	short show_scale;
 	int graph_width;
-	int scaled;
+	int graph_allocated;
+	int scaled;			/* auto adjust maximum */
 	unsigned long first_colour;	// for graph gradient
 	unsigned long last_colour;
 	short font_added;
 	char tempgrad;
+	struct special_t *next;
 };
 
 /* direct access to the registered specials (FIXME: bad encapsulation) */
 extern struct special_t *specials;
 extern int special_count;
 
-extern int default_bar_width;
-extern int default_bar_height;
-#ifdef X11
-extern int default_graph_width;
-extern int default_graph_height;
-extern int default_gauge_width;
-extern int default_gauge_height;
-#endif
-
-/* max number of specials allowed (TODO: use linked list instead) */
-extern int max_specials;
+/* forward declare to avoid mutual inclusion between specials.h and text_object.h */
+struct text_object;
 
 /* scanning special arguments */
-const char *scan_bar(const char *, int *, int *);
-#ifdef X11
-const char *scan_gauge(const char *, int *, int *);
-char *scan_font(const char *);
-char *scan_graph(const char *, int *, int *, unsigned int *,
-                 unsigned int *, unsigned int *, char *, char *);
+const char *scan_bar(struct text_object *, const char *, double);
+const char *scan_gauge(struct text_object *, const char *, double);
+#ifdef BUILD_X11
+void scan_font(struct text_object *, const char *);
+char *scan_graph(struct text_object *, const char *, double);
+void scan_tab(struct text_object *, const char *);
+void scan_stippled_hr(struct text_object *, const char*);
 
 /* printing specials */
-void new_gauge(char *, int, int, int);
-void new_bar(char *, int, int, int);
-void new_font(char *, char *);
-void new_graph(char *, int, int, unsigned int,
-               unsigned int, double, int, int, char, char);
-void new_hr(char *, int);
-void new_stippled_hr(char *, int, int);
-#endif
-void new_bar_in_shell(char *, int, double, int);
-void new_fg(char *, long);
-void new_bg(char *, long);
-void new_outline(char *, long);
-void new_offset(char *, long);
-void new_voffset(char *, long);
-void new_alignr(char *, long);
-void new_alignc(char *, long);
-void new_goto(char *, long);
-void new_tab(char *, int, int);
+void new_font(struct text_object *, char *, int);
+void new_graph(struct text_object *, char *, int, double);
+void new_hr(struct text_object *, char *, int);
+void new_stippled_hr(struct text_object *, char *, int);
+#endif /* BUILD_X11 */
+void new_gauge(struct text_object *, char *, int, double);
+void new_bar(struct text_object *, char *, int, double);
+void new_fg(struct text_object *, char *, int);
+void new_bg(struct text_object *, char *, int);
+void new_outline(struct text_object *, char *, int);
+void new_offset(struct text_object *, char *, int);
+void new_voffset(struct text_object *, char *, int);
+void new_alignr(struct text_object *, char *, int);
+void new_alignc(struct text_object *, char *, int);
+void new_goto(struct text_object *, char *, int);
+void new_tab(struct text_object *, char *, int);
+
+struct special_t *new_special(char *buf, enum special_types t);
 
 #endif /* _SPECIALS_H */
