@@ -1,4 +1,7 @@
-/* Conky, a system monitor, based on torsmo
+/* -*- mode: c++; c-basic-offset: 4; tab-width: 4; indent-tabs-mode: t -*-
+ * vim: ts=4 sw=4 noet ai cindent syntax=cpp
+ *
+ * Conky, a system monitor, based on torsmo
  *
  * Any original torsmo code is licensed under the BSD license
  *
@@ -7,7 +10,7 @@
  * Please see COPYING for details
  *
  * Copyright (c) 2004, Hannu Saransaari and Lauri Hakkarainen
- * Copyright (c) 2005-2009 Brenden Matthews, Philip Kovacs, et. al.
+ * Copyright (c) 2005-2012 Brenden Matthews, Philip Kovacs, et. al.
  *	(see AUTHORS)
  * All rights reserved.
  *
@@ -25,5 +28,15 @@
  *
  */
 
-void init_tailhead(const char* type, const char* arg, struct text_object *obj, void* free_at_crash);
-void print_tailhead(const char* type, struct text_object *obj, char *p, int p_max_size);
+#ifndef _TAILHEAD_H
+#define _TAILHEAD_H
+
+void free_tailhead(struct text_object *);
+void init_tailhead(const char *, const char *, struct text_object *, void *);
+void print_head(struct text_object *, char *, int);
+void print_tail(struct text_object *, char *, int);
+
+void print_lines(struct text_object *, char *, int);
+void print_words(struct text_object *, char *, int);
+
+#endif /* _TAILHEAD_H */
