@@ -1,4 +1,7 @@
-/* tcp-portmon.h - libtcp-portmon hooks protoypes
+/* -*- mode: c++; c-basic-offset: 4; tab-width: 4; indent-tabs-mode: t -*-
+ * vim: ts=4 sw=4 noet ai cindent syntax=cpp
+ *
+ * tcp-portmon.h - libtcp-portmon hooks protoypes
  *
  * Copyright (C) 2008 Phil Sutter <Phil@nwl.cc>
  *
@@ -14,8 +17,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id$
- *
  */
 #ifndef _TCP_PORTMON_H
 #define _TCP_PORTMON_H
@@ -33,10 +34,13 @@ struct tcp_port_monitor_data {
 	int connection_index;
 };
 
-int tcp_portmon_init(const char *, struct tcp_port_monitor_data *);
-int tcp_portmon_action(char *, int, struct tcp_port_monitor_data *);
+/* forward declare to make gcc happy */
+struct text_object;
+
+int tcp_portmon_init(struct text_object *, const char *);
+void tcp_portmon_action(struct text_object *, char *, int);
 int tcp_portmon_update(void);
 int tcp_portmon_clear(void);
-int tcp_portmon_set_max_connections(int);
+void tcp_portmon_free(struct text_object *);
 
 #endif /* _TCP_PORTMON_H */
