@@ -47,10 +47,6 @@
 #ifdef BUILD_XFT
 #include <X11/Xft/Xft.h>
 #endif
-#ifdef BUILD_XSHAPE
-#include <X11/extensions/shape.h>
-#include <X11/extensions/shapeconst.h>
-#endif
 #ifdef BUILD_XINERAMA
 #include <X11/extensions/Xinerama.h>
 #endif
@@ -491,7 +487,8 @@ static void update_workarea(void)
 	workarea[0] = ps->x_org;
 	workarea[1] = ps->y_org;
 	workarea[2] = workarea[0] + ps->width;
-	workarea[3] = workarea[1] + ps->height;
+        workarea[3] = workarea[1] + ps->height;
+        XFree(si);
 
 	DBGP("Fixed xinerama area to: %d %d %d %d", 
 		workarea[0], workarea[1], workarea[2], workarea[3]);
@@ -770,24 +767,6 @@ static void init_window(lua::state &l __attribute__((unused)), bool own)
 			/* allow decorated windows to be given input focus by WM */
 			wmHint.input =
 				TEST_HINT(hints, HINT_UNDECORATED) ? False : True;
-#ifdef BUILD_XSHAPE
-			if (!wmHint.input) {
-				int event_base, error_base;
-				if (XShapeQueryExtension(display, &event_base, &error_base)) {
-					int major_version = 0, minor_version = 0;
-					XShapeQueryVersion(display, &major_version, &minor_version);
-					if ((major_version > 1) || ((major_version == 1) && (minor_version >=1))) {
-						Region empty_region = XCreateRegion();
-						XShapeCombineRegion(display, window.window, ShapeInput, 0, 0, empty_region, ShapeSet);
-						XDestroyRegion(empty_region);
-					} else {
-						NORM_ERR("Input shapes are not supported");
-					}
-				} else {
-					NORM_ERR("No shape extension found");
-				}
-			}
-#endif
 			if (own_window_type.get(l) == TYPE_DOCK || own_window_type.get(l) == TYPE_PANEL) {
 				wmHint.initial_state = WithdrawnState;
 			} else {
